class PastesController < ApplicationController
  before_action :set_paste, only: [:show, :edit, :update, :destroy]

  # GET /pastes
  # GET /pastes.json
  def index
    # Get combined pastes from both SQLite and Elasticsearch
    if params[:search].present?
      # For search, use Elasticsearch only for now
      @pastes = Paste.from_elasticsearch(limit: 20).select do |paste|
        paste.content.downcase.include?(params[:search].downcase) ||
        paste.title.downcase.include?(params[:search].downcase)
      end
    else
      # Show combined recent pastes
      @pastes = Paste.recent_combined(limit: 20)
    end
    
    # Filter by language if specified
    if params[:language].present?
      @pastes = @pastes.select { |paste| paste.language == params[:language] }
    end
    
    # For filter dropdown - combine languages from both sources
    sqlite_languages = Paste.languages
    es_languages = @pastes.map(&:language).uniq.compact
    @languages = (sqlite_languages + es_languages).uniq.sort
    
    # Stats for dashboard
    @total_pastes = Paste.count + get_elasticsearch_count
    @total_languages = @languages.count
  end

  private
  
  def get_elasticsearch_count
    begin
      client = Elasticsearch::Client.new(url: ENV.fetch('ELASTICSEARCH_URL', 'http://pastebinner-elasticsearch:9200'))
      response = client.count(index: 'pastes')
      response['count'] || 0
    rescue => e
      Rails.logger.error "Elasticsearch count error: #{e.message}"
      0
    end
  end

  # GET /pastes/1
  # GET /pastes/1.json
  def show
    # Increment hit counter only for SQLite pastes
    @paste.increment_hits! if @paste.respond_to?(:increment_hits!)
    
    # For raw text display
    if params[:raw] == 'true'
      render plain: @paste.content, content_type: 'text/plain'
      return
    end
  end

  # GET /pastes/new
#  def new
#    @paste = Paste.new
#  end

#  # GET /pastes/1/edit
#  def edit
#  end

  # POST /pastes
  # POST /pastes.json
#  def create
#    @paste = Paste.new(paste_params)
#
#    respond_to do |format|
#      if @paste.save
#        format.html { redirect_to @paste, notice: 'Paste was successfully created.' }
#        format.json { render :show, status: :created, location: @paste }
#      else
#        format.html { render :new }
#        format.json { render json: @paste.errors, status: :unprocessable_entity }
#      end
#    end
#  end

  # PATCH/PUT /pastes/1
  # PATCH/PUT /pastes/1.json
#  def update
#    respond_to do |format|
#      if @paste.update(paste_params)
#        format.html { redirect_to @paste, notice: 'Paste was successfully updated.' }
#        format.json { render :show, status: :ok, location: @paste }
#      else
#        format.html { render :edit }
#        format.json { render json: @paste.errors, status: :unprocessable_entity }
#      end
#    end
#  end

  # DELETE /pastes/1
  # DELETE /pastes/1.json
#  def destroy
#    @paste.destroy
#    respond_to do |format|
#      format.html { redirect_to pastes_url, notice: 'Paste was successfully destroyed.' }
#      format.json { head :no_content }
#    end
#  end

  private
    # Use callbacks to share common setup or constraints between actions.
    def set_paste
      # Try to find in SQLite first
      begin
        @paste = Paste.find(params[:id])
      rescue ActiveRecord::RecordNotFound
        # If not found in SQLite, try Elasticsearch by document ID
        @paste = find_elasticsearch_paste(params[:id])
        unless @paste
          redirect_to pastes_path, alert: 'Paste not found'
        end
      end
    end
    
    def find_elasticsearch_paste(doc_id)
      begin
        client = Elasticsearch::Client.new(url: ENV.fetch('ELASTICSEARCH_URL', 'http://pastebinner-elasticsearch:9200'))
        response = client.get(index: 'pastes', id: doc_id)
        Paste.elasticsearch_paste_to_object(response)
      rescue => e
        Rails.logger.error "Elasticsearch get error: #{e.message}"
        nil
      end
    end

    # Never trust parameters from the scary internet, only allow the white list through.
    def paste_params
      params.require(:paste).permit(:title, :content, :language, :expires_at)
    end
end
