require 'elasticsearch/model'
require 'ostruct'

class Paste < ApplicationRecord
  include Elasticsearch::Model
  # include Elasticsearch::Model::Callbacks  # Temporarily disabled for seeding

  # Pagination
  paginates_per 20

  # Validations
  validates :content, presence: true
  validates :paste_key, uniqueness: true, allow_blank: true

  # Callbacks
  before_save :set_defaults
  before_save :detect_language
  before_save :calculate_size

  # Scopes
  scope :by_language, ->(lang) { where(language: lang) if lang.present? }
  scope :recent, -> { order(created_at: :desc) }
  scope :popular, -> { order(hits: :desc) }

  # Class methods
  def self.search_content(query)
    return all if query.blank?
    
    __elasticsearch__.search({
      query: {
        multi_match: {
          query: query,
          fields: ['content', 'title']
        }
      },
      highlight: {
        pre_tags: ['<mark>'],
        post_tags: ['</mark>'],
        fields: {
          content: {}
        }
      }
    })
  end

  def self.languages
    distinct.pluck(:language).compact.sort
  end

  # Fetch pastes from Elasticsearch
  def self.from_elasticsearch(limit: 20, offset: 0)
    begin
      client = Elasticsearch::Client.new(url: ENV.fetch('ELASTICSEARCH_URL', 'http://pastebinner-elasticsearch:9200'))
      
      response = client.search(
        index: 'pastes',
        body: {
          size: limit,
          from: offset,
          sort: [{ '_score': { order: 'desc' } }],
          query: { match_all: {} }
        }
      )
      
      response['hits']['hits'].map do |hit|
        elasticsearch_paste_to_object(hit)
      end
    rescue => e
      Rails.logger.error "Elasticsearch error: #{e.message}"
      []
    end
  end

  # Convert Elasticsearch hit to paste-like object
  def self.elasticsearch_paste_to_object(hit)
    source = hit['_source']
    paste_id = extract_paste_key_from_metadata(source['paste_metadata'])
    
    OpenStruct.new(
      id: hit['_id'],
      title: "Scraped Paste #{paste_id}",
      content: source['paste_text'] || '',
      language: detect_language_from_content(source['paste_text']),
      created_at: Time.current,
      updated_at: Time.current,
      hits: 0,
      size: (source['paste_text'] || '').length,
      paste_key: paste_id,
      source: 'elasticsearch'
    )
  end

  # Extract paste key from metadata array
  def self.extract_paste_key_from_metadata(metadata)
    return 'unknown' unless metadata.is_a?(Array) && metadata.length >= 2
    
    url = metadata[1] # The scrape URL
    if url =~ /\/api_scrape_item\.php\?i=([A-Za-z0-9]+)/
      $1
    else
      'unknown'
    end
  end

  # Simple language detection based on content
  def self.detect_language_from_content(content)
    return 'text' if content.blank?
    
    case content
    when /^\s*<\?php/m then 'php'
    when /^\s*#!/m then 'bash'
    when /function\s*\(/m, /=>\s*{/m, /console\.log/m then 'javascript'
    when /def\s+\w+/m, /import\s+\w+/m then 'python'
    when /<html/m, /<DOCTYPE/m then 'html'
    when /SELECT\s+/im, /INSERT\s+/im, /UPDATE\s+/im then 'sql'
    when /{[^}]*}/m, /\[[^\]]*\]/m then 'json'
    else 'text'
    end
  end

  # Get combined pastes from both sources
  def self.recent_combined(limit: 20)
    # Get from SQLite
    sqlite_pastes = recent.limit(limit/2).to_a
    
    # Get from Elasticsearch  
    es_pastes = from_elasticsearch(limit: limit/2)
    
    # Combine and return
    (sqlite_pastes + es_pastes).sort_by { |p| p.created_at || Time.current }.reverse.first(limit)
  end

  # Instance methods
  def display_title
    title.present? ? title : "Paste ##{id}"
  end

  def short_content(limit = 200)
    return '' if content.blank?
    content.length > limit ? "#{content[0..limit]}..." : content
  end

  def formatted_size
    return '0 B' if size.nil? || size.zero?
    
    units = %w[B KB MB GB]
    base = 1024
    exp = (Math.log(size) / Math.log(base)).to_i
    exp = [exp, units.size - 1].min
    
    "%.1f %s" % [size.to_f / base**exp, units[exp]]
  end

  def language_display
    language.present? ? language.humanize : 'Plain Text'
  end

  def increment_hits!
    increment!(:hits)
  end

  def expired?
    expires_at && expires_at < Time.current
  end

  def prism_language
    # Map common languages to Prism.js language identifiers
    language_mappings = {
      'javascript' => 'javascript',
      'js' => 'javascript',
      'python' => 'python',
      'py' => 'python',
      'ruby' => 'ruby',
      'rb' => 'ruby',
      'java' => 'java',
      'cpp' => 'cpp',
      'c++' => 'cpp',
      'c' => 'c',
      'html' => 'html',
      'css' => 'css',
      'php' => 'php',
      'sql' => 'sql',
      'json' => 'json',
      'xml' => 'xml',
      'yaml' => 'yaml',
      'yml' => 'yaml',
      'bash' => 'bash',
      'shell' => 'bash',
      'go' => 'go',
      'rust' => 'rust',
      'typescript' => 'typescript',
      'ts' => 'typescript'
    }
    
    return language_mappings[language.to_s.downcase] if language.present?
    'text'
  end

  private

  def set_defaults
    self.paste_key = SecureRandom.hex(8) if paste_key.blank?
    self.hits ||= 0
    self.title = "Untitled" if title.blank?
  end

  def detect_language
    return if content.blank?
    
    # Simple language detection based on content patterns
    case content
    when /def\s+\w+\s*\(/
      self.language = 'python' if language.blank?
    when /function\s+\w+\s*\(/
      self.language = 'javascript' if language.blank?
    when /class\s+\w+/
      self.language = 'java' if language.blank?
    when /<\?php/
      self.language = 'php' if language.blank?
    when /<!DOCTYPE|<html/
      self.language = 'html' if language.blank?
    when /SELECT|INSERT|UPDATE|DELETE/i
      self.language = 'sql' if language.blank?
    else
      self.language = 'text' if language.blank?
    end
  end

  def calculate_size
    self.size = content.present? ? content.bytesize : 0
  end
end
