# Sample paste data for testing the web interface

puts "Creating sample pastes..."

# Python sample
Paste.create!(
  title: "Hello World Python",
  content: <<~PYTHON,
    def hello_world():
        """A simple greeting function"""
        name = input("What's your name? ")
        print(f"Hello, {name}!")
        return name

    if __name__ == "__main__":
        hello_world()
  PYTHON
  language: "python"
)

# JavaScript sample  
Paste.create!(
  title: "React Component",
  content: <<~JAVASCRIPT,
    import React, { useState, useEffect } from 'react';

    const UserProfile = ({ userId }) => {
      const [user, setUser] = useState(null);
      const [loading, setLoading] = useState(true);

      useEffect(() => {
        fetch(`/api/users/${userId}`)
          .then(response => response.json())
          .then(userData => {
            setUser(userData);
            setLoading(false);
          })
          .catch(error => {
            console.error('Error fetching user:', error);
            setLoading(false);
          });
      }, [userId]);

      if (loading) return <div>Loading...</div>;
      if (!user) return <div>User not found</div>;

      return (
        <div className="user-profile">
          <h1>{user.name}</h1>
          <p>{user.email}</p>
          <p>Joined: {new Date(user.createdAt).toLocaleDateString()}</p>
        </div>
      );
    };

    export default UserProfile;
  JAVASCRIPT
  language: "javascript"
)

# Ruby sample
Paste.create!(
  title: "Rails Migration",
  content: <<~RUBY,
    class CreateUsersTable < ActiveRecord::Migration[6.1]
      def change
        create_table :users do |t|
          t.string :name, null: false
          t.string :email, null: false, index: { unique: true }
          t.string :password_digest
          t.text :bio
          t.boolean :active, default: true
          t.timestamps
        end

        add_index :users, :name
        add_index :users, :created_at
      end
    end
  RUBY
  language: "ruby"
)

# SQL sample
Paste.create!(
  title: "Database Query",
  content: <<~SQL,
    SELECT 
        u.name,
        u.email,
        COUNT(p.id) as post_count,
        AVG(p.views) as avg_views
    FROM users u
    LEFT JOIN posts p ON u.id = p.user_id
    WHERE u.active = true
        AND u.created_at >= '2024-01-01'
    GROUP BY u.id, u.name, u.email
    HAVING COUNT(p.id) > 5
    ORDER BY avg_views DESC
    LIMIT 20;
  SQL
  language: "sql"
)

# HTML sample
Paste.create!(
  title: "Landing Page Template",
  content: <<~HTML,
    <!DOCTYPE html>
    <html lang="en">
    <head>
        <meta charset="UTF-8">
        <meta name="viewport" content="width=device-width, initial-scale=1.0">
        <title>My Awesome App</title>
        <link rel="stylesheet" href="styles.css">
    </head>
    <body>
        <header class="hero">
            <nav class="navbar">
                <div class="logo">MyApp</div>
                <ul class="nav-links">
                    <li><a href="#features">Features</a></li>
                    <li><a href="#pricing">Pricing</a></li>
                    <li><a href="#contact">Contact</a></li>
                </ul>
            </nav>
            
            <div class="hero-content">
                <h1>Build Amazing Things</h1>
                <p>The fastest way to create and deploy your applications</p>
                <button class="cta-button">Get Started</button>
            </div>
        </header>

        <section id="features" class="features">
            <h2>Why Choose Us?</h2>
            <div class="feature-grid">
                <div class="feature">
                    <h3>Fast</h3>
                    <p>Lightning-fast performance</p>
                </div>
                <div class="feature">
                    <h3>Secure</h3>
                    <p>Enterprise-grade security</p>
                </div>
                <div class="feature">
                    <h3>Scalable</h3>
                    <p>Grows with your business</p>
                </div>
            </div>
        </section>
    </body>
    </html>
  HTML
  language: "html"
)

# CSS sample
Paste.create!(
  title: "Modern CSS Grid Layout",
  content: <<~CSS,
    .container {
      display: grid;
      grid-template-columns: repeat(auto-fit, minmax(300px, 1fr));
      gap: 2rem;
      padding: 2rem;
      max-width: 1200px;
      margin: 0 auto;
    }

    .card {
      background: white;
      border-radius: 12px;
      box-shadow: 0 4px 6px rgba(0, 0, 0, 0.1);
      overflow: hidden;
      transition: transform 0.3s ease, box-shadow 0.3s ease;
    }

    .card:hover {
      transform: translateY(-5px);
      box-shadow: 0 8px 25px rgba(0, 0, 0, 0.15);
    }

    .card-header {
      padding: 1.5rem;
      background: linear-gradient(135deg, #667eea 0%, #764ba2 100%);
      color: white;
    }

    .card-title {
      margin: 0;
      font-size: 1.25rem;
      font-weight: 600;
    }

    .card-body {
      padding: 1.5rem;
    }

    @media (max-width: 768px) {
      .container {
        grid-template-columns: 1fr;
        padding: 1rem;
      }
    }
  CSS
  language: "css"
)

# JSON sample
Paste.create!(
  title: "API Response Example",
  content: <<~JSON,
    {
      "status": "success",
      "data": {
        "users": [
          {
            "id": 1,
            "username": "john_doe",
            "email": "john@example.com",
            "profile": {
              "firstName": "John",
              "lastName": "Doe",
              "avatar": "https://example.com/avatars/john.jpg",
              "bio": "Software developer passionate about clean code"
            },
            "preferences": {
              "theme": "dark",
              "notifications": {
                "email": true,
                "push": false,
                "sms": false
              },
              "privacy": {
                "profilePublic": true,
                "showEmail": false
              }
            },
            "stats": {
              "postsCount": 42,
              "followersCount": 156,
              "followingCount": 89
            },
            "createdAt": "2024-01-15T10:30:00Z",
            "lastLoginAt": "2024-01-26T14:45:23Z"
          }
        ],
        "pagination": {
          "currentPage": 1,
          "totalPages": 10,
          "totalItems": 100,
          "itemsPerPage": 10
        }
      },
      "meta": {
        "timestamp": "2024-01-26T15:00:00Z",
        "version": "1.0",
        "requestId": "req_123456789"
      }
    }
  JSON
  language: "json"
)

puts "Created #{Paste.count} sample pastes!"
puts "You can now browse them at http://localhost:3000"
